from decimal import Decimal
import hashlib
import logging
import urllib
import urllib2
from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import ugettext_lazy as _
import time
from getpaid import signals
from getpaid.backends import PaymentProcessorBase
from getpaid.backends.payu.tasks import get_payment_status_task, accept_payment

logger = logging.getLogger('getpaid.backends.payu')


class PayUTransactionStatus:
    NEW = 1
    CANCELED = 2
    REJECTED = 3
    STARTED = 4
    AWAITING = 5
    REJECTED_AFTER_CANCEL = 7
    FINISHED = 99
    ERROR = 888


class PaymentProcessor(PaymentProcessorBase):
    BACKEND = 'getpaid.backends.payu'
    BACKEND_NAME = _('PayU')
    BACKEND_ACCEPTED_CURRENCY = ('PLN', )
    BACKEND_LOGO_URL = 'getpaid/backends/payu/payu_logo.png'

    _GATEWAY_URL = 'https://www.platnosci.pl/paygw/'
    _ACCEPTED_LANGS = ('pl', 'en')
    _REQUEST_SIG_FIELDS = ('pos_id', 'pay_type', 'session_id', 'pos_auth_key',
                           'amount', 'desc', 'desc2', 'trsDesc', 'order_id', 'first_name', 'last_name',
                           'payback_login', 'street', 'street_hn', 'street_an', 'city', 'post_code',
                           'country', 'email', 'phone', 'language', 'client_ip', 'ts' )
    _ONLINE_SIG_FIELDS = ('pos_id', 'session_id', 'ts',)
    _GET_SIG_FIELDS = ('pos_id', 'session_id', 'ts',)
    _GET_RESPONSE_SIG_FIELDS = (
        'trans_pos_id', 'trans_session_id', 'trans_order_id', 'trans_status', 'trans_amount', 'trans_desc', 'trans_ts',)
    _GET_ACCEPT_SIG_FIELDS = ('trans_pos_id', 'trans_session_id', 'trans_ts',)

    @staticmethod
    def compute_sig(params, fields, key):
        text = ''
        for field in fields:
            text += unicode(params.get(field, '')).encode('utf-8')
        text += key
        return hashlib.md5(text).hexdigest()

    @staticmethod
    def online(pos_id, session_id, ts, sig):
        params = {'pos_id': pos_id, 'session_id': session_id, 'ts': ts, 'sig': sig}

        key2 = PaymentProcessor.get_backend_setting('key2')
        if sig != PaymentProcessor.compute_sig(params, PaymentProcessor._ONLINE_SIG_FIELDS, key2):
            logger.warning('Got message with wrong sig, %s' % str(params))
            return 'SIG ERR'

        try:
            params['pos_id'] = int(params['pos_id'])
        except ValueError:
            return 'POS_ID ERR'
        if params['pos_id'] != int(PaymentProcessor.get_backend_setting('pos_id')):
            return 'POS_ID ERR'

        try:
            payment_id, session = session_id.split(':')
        except ValueError:
            logger.warning('Got message with wrong session_id, %s' % str(params))
            return 'SESSION_ID ERR'

        get_payment_status_task.delay(payment_id, session_id)
        return 'OK'

    def get_gateway_url(self, request):
        """
        Routes a payment to Gateway, should return URL for redirection.

        """
        params = {
            'pos_id': PaymentProcessor.get_backend_setting('pos_id'),
            'pos_auth_key': PaymentProcessor.get_backend_setting('pos_auth_key'),
            'desc': self.get_order_description(self.payment, self.payment.order),
        }

        user_data = {
            'email': None,
            'lang': None,
        }

        signals.user_data_query.send(sender=None, order=self.payment.order, user_data=user_data)
        if user_data['email']:
            params['email'] = user_data['email']

        if user_data['lang'] and user_data['lang'].lower() in PaymentProcessor._ACCEPTED_LANGS:
            params['language'] = user_data['lang'].lower()
        elif PaymentProcessor.get_backend_setting('lang', False) and \
                        PaymentProcessor.get_backend_setting('lang').lower() in PaymentProcessor._ACCEPTED_LANGS:
            params['language'] = PaymentProcessor.get_backend_setting('lang').lower()

        key1 = PaymentProcessor.get_backend_setting('key1')

        signing = PaymentProcessor.get_backend_setting('signing', True)
        testing = PaymentProcessor.get_backend_setting('testing', False)

        if testing:
            # Switch to testing mode, where payment method is set to "test payment"->"t"
            # Warning: testing mode need to be enabled also in payu.pl system for this POS
            params['pay_type'] = 't'

        # Here we put payment.pk as we can get order through payment model
        params['order_id'] = self.payment.pk

        # amount is number of Grosz, not PLN
        params['amount'] = int(self.payment.amount * 100)

        params['session_id'] = "%d:%s" % (self.payment.pk, str(time.time()))

        #Warning: please make sure that this header actually has client IP
        #         rather then web server proxy IP in your WSGI environment
        params['client_ip'] = request.META['REMOTE_ADDR']

        if signing:
            params['ts'] = time.time()
            params['sig'] = PaymentProcessor.compute_sig(params, self._REQUEST_SIG_FIELDS, key1)

        if PaymentProcessor.get_backend_setting('method', 'get').lower() == 'post':
            logger.info(u'New payment using POST: %s' % params)
            return self._GATEWAY_URL + 'UTF/NewPayment', 'POST', params
        elif PaymentProcessor.get_backend_setting('method', 'get').lower() == 'get':
            logger.info(u'New payment using GET: %s' % params)
            for key in params.keys():
                params[key] = unicode(params[key]).encode('utf-8')
            return self._GATEWAY_URL + 'UTF/NewPayment?' + urllib.urlencode(params), 'GET', {}
        else:
            logger.error(u'New payment raises error - bad HTTP method')
            raise ImproperlyConfigured('PayU payment backend accepts only GET or POST')

    def get_payment_status(self, session_id):
        params = {'pos_id': PaymentProcessor.get_backend_setting('pos_id'), 'session_id': session_id, 'ts': time.time()}
        key1 = PaymentProcessor.get_backend_setting('key1')
        key2 = PaymentProcessor.get_backend_setting('key2')

        params['sig'] = PaymentProcessor.compute_sig(params, self._GET_SIG_FIELDS, key1)

        for key in params.keys():
            params[key] = unicode(params[key]).encode('utf-8')

        data = urllib.urlencode(params)
        url = self._GATEWAY_URL + 'UTF/Payment/get/txt'
        request = urllib2.Request(url, data)
        response = urllib2.urlopen(request)
        response_params = PaymentProcessor._parse_text_response(response.read().decode('utf-8'))

        if not response_params['status'] == 'OK':
            logger.warning(u'Payment status error: %s' % response_params)
            return

        if PaymentProcessor.compute_sig(response_params, self._GET_RESPONSE_SIG_FIELDS, key2) == response_params[
            'trans_sig']:
            if not (int(response_params['trans_pos_id']) == int(params['pos_id']) or int(
                    response_params['trans_order_id']) == self.payment.pk):
                logger.error(u'Payment status wrong pos_id and/or order id: %s' % response_params)
                return

            logger.info(u'Fetching payment status: %s' % response_params)

            self.payment.external_id = response_params['trans_id']

            status = int(response_params['trans_status'])
            if status in (PayUTransactionStatus.AWAITING, PayUTransactionStatus.FINISHED):

                if self.payment.on_success(Decimal(response_params['trans_amount']) / Decimal('100')):
                    #fully paid
                    if status == PayUTransactionStatus.AWAITING:
                        accept_payment.delay(self.payment.id, session_id)

            elif status in (PayUTransactionStatus.CANCELED,
                            PayUTransactionStatus.ERROR,
                            PayUTransactionStatus.REJECTED,
                            PayUTransactionStatus.REJECTED_AFTER_CANCEL):
                self.payment.on_failure()

        else:
            logger.error(u'Payment status wrong response signature: %s' % response_params)

    def accept_payment(self, session_id):
        params = {'pos_id': PaymentProcessor.get_backend_setting('pos_id'), 'session_id': session_id, 'ts': time.time()}
        key1 = PaymentProcessor.get_backend_setting('key1')
        key2 = PaymentProcessor.get_backend_setting('key2')
        params['sig'] = PaymentProcessor.compute_sig(params, self._GET_SIG_FIELDS, key1)
        for key in params.keys():
            params[key] = unicode(params[key]).encode('utf-8')
        data = urllib.urlencode(params)
        url = self._GATEWAY_URL + 'UTF/Payment/confirm/txt'
        request = urllib2.Request(url, data)
        response = urllib2.urlopen(request)
        response_params = PaymentProcessor._parse_text_response(response.read().decode('utf-8'))
        if response_params['status'] == 'OK':
            if PaymentProcessor.compute_sig(response_params, self._GET_ACCEPT_SIG_FIELDS, key2) != response_params[
                'trans_sig']:
                logger.error(u'Wrong signature for Payment/confirm response: %s' % response_params)
                return
            if int(response_params['trans_pos_id']) != int(params['pos_id']):
                logger.error(u'Wrong pos_id for Payment/confirm response: %s' % response_params)
                return

            logger.info(u'Payment accepted: %s' % response_params)
        else:
            logger.warning(u'Payment not accepted, error: %s' % response_params)


    @staticmethod
    def _parse_text_response(text):
        """
        Parses inputs like:
        variable : some value
        variable2 : 123.44
        into dict
        """
        return dict(
            map(lambda (k, v): (k.rstrip(), v.lstrip()),
                filter(
                    lambda l: len(l) == 2,
                    map(lambda l: l.split(':', 1),
                        text.splitlines())
                )
            )
        )


# Author: Krzysztof Dorosz <cypreess@gmail.com>
#
# Disclaimer:
# Writing and open sourcing this backend was kindly funded by Issue Stand
# http://issuestand.com/
#

from decimal import Decimal
import hashlib
import logging
import time
import urllib
import urllib2
import datetime

from django.contrib.sites.models import Site
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from pytz import utc

from getpaid import signals
from getpaid.backends import PaymentProcessorBase
from getpaid.backends.przelewy24.tasks import get_payment_status_task

logger = logging.getLogger('getpaid.backends.przelewy24')


class PaymentProcessor(PaymentProcessorBase):
    BACKEND = 'getpaid.backends.przelewy24'
    BACKEND_NAME = _('Przelewy24')
    BACKEND_ACCEPTED_CURRENCY = ('PLN', )
    BACKEND_LOGO_URL = 'getpaid/backends/przelewy24/przelewy24_logo.png'

    _GATEWAY_URL = 'https://secure.przelewy24.pl/index.php'
    _SANDBOX_GATEWAY_URL = 'https://sandbox.przelewy24.pl/index.php'

    _GATEWAY_CONFIRM_URL = 'https://secure.przelewy24.pl/transakcja.php'
    _SANDBOX_GATEWAY_CONFIRM_URL = 'https://sandbox.przelewy24.pl/transakcja.php'

    _ACCEPTED_LANGS = ('pl', 'en', 'es', 'de', 'it')
    _REQUEST_SIG_FIELDS = ('p24_session_id', 'p24_id_sprzedawcy', 'p24_kwota', 'crc')
    _SUCCESS_RETURN_SIG_FIELDS = ('p24_session_id', 'p24_order_id', 'p24_kwota', 'crc')
    _STATUS_SIG_FIELDS = ('p24_session_id', 'p24_order_id', 'p24_kwota', 'crc')

    @staticmethod
    def compute_sig(params, fields, crc):
        params = params.copy()
        params.update({'crc': crc})
        text = "|".join(map(lambda field: unicode(params.get(field, '')).encode('utf-8'), fields))
        return hashlib.md5(text).hexdigest()

    @staticmethod
    def on_payment_status_change(p24_session_id, p24_order_id, p24_kwota, p24_order_id_full, p24_crc):
        params = {
            'p24_session_id': p24_session_id,
            'p24_order_id': p24_order_id,
            'p24_kwota': p24_kwota,
            'p24_order_id_full': p24_order_id_full,
            'p24_crc': p24_crc,
        }
        crc = PaymentProcessor.get_backend_setting('crc')
        if p24_crc != PaymentProcessor.compute_sig(params, PaymentProcessor._SUCCESS_RETURN_SIG_FIELDS,
                                                   crc):
            logger.warning('Success return call has wrong crc %s' % str(params))
            return False

        payment_id = p24_session_id.split(':')[0]
        get_payment_status_task.delay(payment_id, p24_session_id, p24_order_id, p24_kwota)
        return True

    def get_payment_status(self, p24_session_id, p24_order_id, p24_kwota):
        params = {
            'p24_session_id': p24_session_id,
            'p24_order_id': p24_order_id,
            'p24_id_sprzedawcy': PaymentProcessor.get_backend_setting('id'),
            'p24_kwota': p24_kwota,
        }
        crc = PaymentProcessor.get_backend_setting('crc')
        params['p24_crc'] = PaymentProcessor.compute_sig(params, self._STATUS_SIG_FIELDS, crc)

        for key in params.keys():
            params[key] = unicode(params[key]).encode('utf-8')

        data = urllib.urlencode(params)
        url = self._SANDBOX_GATEWAY_CONFIRM_URL if PaymentProcessor.get_backend_setting('sandbox',
                                                                                        False) else self._GATEWAY_CONFIRM_URL

        self.payment.external_id = params['p24_order_id']

        request = urllib2.Request(url, data)
        try:
            response = urllib2.urlopen(request).read()
        except Exception:
            logger.exception('Error while getting payment status change %s data=%s' % (url, str(params)))
            return

        response_list = filter(lambda ll: ll, map(lambda l: l.strip(), response.splitlines()))

        if len(response_list) >= 2 and response_list[0] == 'RESULT' and response_list[1] == 'TRUE':
            logger.info('Payment accepted %s' % str(params))
            self.payment.amount_paid = Decimal(p24_kwota) / Decimal('100')
            self.payment.paid_on = datetime.datetime.utcnow().replace(tzinfo=utc)
            if self.payment.amount_paid >= self.payment.amount:
                self.payment.change_status('paid')
            else:
                self.payment.change_status('partially_paid')
        else:
            logger.warning('Payment rejected for data=%s: "%s"' % (str(params), response))
            self.payment.change_status('failed')

    def get_gateway_url(self, request):
        """
        Routes a payment to Gateway, should return URL for redirection.

        """
        params = {
            'p24_id_sprzedawcy': PaymentProcessor.get_backend_setting('id'),
            'p24_opis': self.get_order_description(self.payment, self.payment.order),
            'p24_session_id': "%s:%s:%s" % (self.payment.pk, self.BACKEND, time.time()),
            'p24_kwota': int(self.payment.amount * 100),
            'p24_email': None,

        }

        user_data = {
            'email': None,
            'lang': None,
        }
        signals.user_data_query.send(sender=None, order=self.payment.order, user_data=user_data)
        if user_data['email']:
            params['p24_email'] = user_data['email']

        if user_data['lang'] and user_data['lang'].lower() in PaymentProcessor._ACCEPTED_LANGS:
            params['p24_language'] = user_data['lang'].lower()
        elif PaymentProcessor.get_backend_setting('lang', False) and PaymentProcessor.get_backend_setting(
                'lang').lower() in PaymentProcessor._ACCEPTED_LANGS:
            params['p24_language'] = PaymentProcessor.get_backend_setting('lang').lower()

        params['p24_crc'] = self.compute_sig(params, self._REQUEST_SIG_FIELDS,
                                             PaymentProcessor.get_backend_setting('crc'))

        current_site = Site.objects.get_current()
        use_ssl = PaymentProcessor.get_backend_setting('ssl_return', False)

        params['p24_return_url_ok'] = ('https://' if use_ssl else 'http://') + current_site.domain + reverse(
            'getpaid-przelewy24-success', kwargs={'pk': self.payment.pk})
        params['p24_return_url_error'] = ('https://' if use_ssl else 'http://') + current_site.domain + reverse(
            'getpaid-przelewy24-failure', kwargs={'pk': self.payment.pk})

        if params['p24_email'] is None:
            raise ImproperlyConfigured(
                '%s requires filling `email` field for payment (you need to handle `user_data_query` signal)' % self.BACKEND)

        return self._SANDBOX_GATEWAY_URL if PaymentProcessor.get_backend_setting('sandbox',
                                                                                 False) else self._GATEWAY_URL, 'POST', params



from decimal import Decimal
import hashlib
import logging
import urllib
import datetime
from django.contrib.sites.models import Site
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse
from django.db.models.loading import get_model
from django.utils.timezone import utc
from django.utils.translation import ugettext_lazy as _
from getpaid import signals
from getpaid.backends import PaymentProcessorBase

logger = logging.getLogger('getpaid.backends.transferuj')


class PaymentProcessor(PaymentProcessorBase):
    BACKEND = 'getpaid.backends.transferuj'
    BACKEND_NAME = _('Transferuj.pl')
    BACKEND_ACCEPTED_CURRENCY = ('PLN', )
    BACKEND_LOGO_URL = 'getpaid/backends/transferuj/transferuj_logo.png'

    _GATEWAY_URL = 'https://secure.transferuj.pl'
    _REQUEST_SIG_FIELDS = ('id', 'kwota', 'crc',)
    _ALLOWED_IP = ('195.149.229.109', )

    _ONLINE_SIG_FIELDS = ('id', 'tr_id', 'tr_amount', 'tr_crc', )
    _ACCEPTED_LANGS = ('pl', 'en', 'de')

    @staticmethod
    def compute_sig(params, fields, key):
        text = ''
        for field in fields:
            text += unicode(params.get(field, '')).encode('utf-8')
        text += key
        return hashlib.md5(text).hexdigest()

    @staticmethod
    def online(ip, id, tr_id, tr_date, tr_crc, tr_amount, tr_paid, tr_desc, tr_status, tr_error, tr_email, md5sum):

        allowed_ip = PaymentProcessor.get_backend_setting('allowed_ip', PaymentProcessor._ALLOWED_IP)

        if len(allowed_ip) != 0 and ip not in allowed_ip:
            logger.warning('Got message from not allowed IP %s' % str(allowed_ip))
            return 'IP ERR'

        params = {'id': id, 'tr_id': tr_id, 'tr_amount': tr_amount, 'tr_crc': tr_crc}
        key = PaymentProcessor.get_backend_setting('key')

        if md5sum != PaymentProcessor.compute_sig(params, PaymentProcessor._ONLINE_SIG_FIELDS, key):
            logger.warning('Got message with wrong sig, %s' % str(params))
            return 'SIG ERR'

        if int(id) != int(PaymentProcessor.get_backend_setting('id')):
            logger.warning('Got message with wrong id, %s' % str(params))
            return 'ID ERR'

        Payment = get_model('getpaid', 'Payment')
        try:
            payment = Payment.objects.select_related('order').get(pk=int(tr_crc))
        except (Payment.DoesNotExist, ValueError):
            logger.error('Got message with CRC set to non existing Payment, %s' % str(params))
            return 'CRC ERR'

        logger.info('Incoming payment: id=%s, tr_id=%s, tr_date=%s, tr_crc=%s, tr_amount=%s, tr_paid=%s, tr_desc=%s, tr_status=%s, tr_error=%s, tr_email=%s' % (id, tr_id, tr_date, tr_crc, tr_amount, tr_paid, tr_desc, tr_status, tr_error, tr_email))

        payment.external_id = tr_id
        payment.description = tr_email

        if tr_status == 'TRUE':
            # Due to Transferuj documentation, we need to check if amount is correct
            payment.amount_paid = Decimal(tr_paid)
            payment.paid_on = datetime.datetime.utcnow().replace(tzinfo=utc)
            if payment.amount <= Decimal(tr_paid):
                # Amount is correct or it is overpaid
                payment.change_status('paid')
            else:
                payment.change_status('partially_paid')
        elif payment.status != 'paid':
            payment.change_status('failed')

        return 'TRUE'

    def get_gateway_url(self, request):
        """
        Routes a payment to Gateway, should return URL for redirection.

        """
        params = {
            'id': PaymentProcessor.get_backend_setting('id'),
            'opis': self.get_order_description(self.payment, self.payment.order),
        }

        user_data = {
            'email': None,
            'lang': None,
        }

        signals.user_data_query.send(sender=None, order=self.payment.order, user_data=user_data)

        if user_data['lang'] and user_data['lang'].lower() in PaymentProcessor._ACCEPTED_LANGS:
            params['jezyk'] = user_data['lang'].lower()
        elif PaymentProcessor.get_backend_setting('lang', False) and\
                PaymentProcessor.get_backend_setting('lang').lower() in PaymentProcessor._ACCEPTED_LANGS:
            params['jezyk'] = PaymentProcessor.get_backend_setting('lang').lower()

        if user_data['email']:
            params['email'] = user_data['email']

        key = PaymentProcessor.get_backend_setting('key')

        signing = PaymentProcessor.get_backend_setting('signing', True)

        # Here we put payment.pk as we can get order through payment model
        params['crc'] = self.payment.pk

        # amount is  in format XXX.YY PLN
        params['kwota'] = str(self.payment.amount)

        if signing:
            params['md5sum'] = PaymentProcessor.compute_sig(params, self._REQUEST_SIG_FIELDS, key)

        current_site = Site.objects.get_current()

        if PaymentProcessor.get_backend_setting('force_ssl_online', False):
            params['wyn_url'] = 'https://' + current_site.domain + reverse('getpaid-transferuj-online')
        else:
            params['wyn_url'] = 'http://' + current_site.domain + reverse('getpaid-transferuj-online')

        if PaymentProcessor.get_backend_setting('force_ssl_return', False):
            params['pow_url'] = 'https://' + current_site.domain + reverse('getpaid-transferuj-success', kwargs={'pk': self.payment.pk})
            params['pow_url_blad'] = 'https://' + current_site.domain + reverse('getpaid-transferuj-failure', kwargs={'pk': self.payment.pk})
        else:
            params['pow_url'] = 'http://' + current_site.domain + reverse('getpaid-transferuj-success', kwargs={'pk': self.payment.pk})
            params['pow_url_blad'] = 'http://' + current_site.domain + reverse('getpaid-transferuj-failure', kwargs={'pk': self.payment.pk})

        if PaymentProcessor.get_backend_setting('method', 'get').lower() == 'post':
            return self._GATEWAY_URL, 'POST', params
        elif PaymentProcessor.get_backend_setting('method', 'get').lower() == 'get':
            for key in params.keys():
                params[key] = unicode(params[key]).encode('utf-8')
            return self._GATEWAY_URL + '?' + urllib.urlencode(params), "GET", {}
        else:
            raise ImproperlyConfigured('Transferuj.pl payment backend accepts only GET or POST')



from django.contrib.sites.models import Site
from django.core.management.base import BaseCommand
from django.core.urlresolvers import reverse
from getpaid.backends.payu import PaymentProcessor


class Command(BaseCommand):
    help = 'Display URL path for PayU Online URL configuration'

    def handle(self, *args, **options):

        current_site = Site.objects.get_current()

        self.stdout.write('Login to PayU configuration page and setup following links:\n\n')
        self.stdout.write(' * Success URL: http://%s%s\n                https://%s%s\n\n' % (
            current_site.domain,
            reverse('getpaid-payu-success', kwargs={'pk': 1234}).replace('1234', '%orderId%'),
            current_site.domain,
            reverse('getpaid-payu-success', kwargs={'pk': 1234}).replace('1234', '%orderId%'),

            )
        )

        self.stdout.write(' * Failure URL: http://%s%s\n                https://%s%s\n\n' % (
            current_site.domain,
            reverse('getpaid-payu-failure', kwargs={'pk': 1234, 'error': 9999}).replace('1234', r'%orderId%').replace('9999', r'%error%'),
            current_site.domain,
            reverse('getpaid-payu-failure', kwargs={'pk': 1234, 'error': 9999}).replace('1234', r'%orderId%').replace('9999', r'%error%'),
            )

        )

        self.stdout.write(' * Online  URL: http://%s%s\n                https://%s%s\n\n' % (
            current_site.domain,
            reverse('getpaid-payu-online'),
            current_site.domain,
            reverse('getpaid-payu-online'),
            )
        )

        self.stdout.write('To change domain name please edit Sites settings. Don\'t forget to setup your web server to accept https connection in order to use secure links.\n')
        if PaymentProcessor.get_backend_setting('testing', False):
            self.stdout.write('\nTesting mode is ON\nPlease be sure that you enabled testing payments in PayU configuration page.\n')
        if PaymentProcessor.get_backend_setting('signing', False):
            self.stdout.write('\nRequest signing is ON\n * Please be sure that you enabled signing payments in PayU configuration page.\n')



from django.conf.urls import patterns, url
from django.views.decorators.csrf import csrf_exempt
from getpaid.backends.dotpay.views import ReturnView
from getpaid.backends.dotpay.views import OnlineView

urlpatterns = patterns('',
    url(r'^online/$', csrf_exempt(OnlineView.as_view()), name='getpaid-dotpay-online'),
    url(r'^return/(?P<pk>\d+)/$', csrf_exempt(ReturnView.as_view()), name='getpaid-dotpay-return'),

)



"""
This file demonstrates writing tests using the unittest module. These will pass
when you run "manage.py test".

Replace this with more appropriate tests for your application.
"""

from decimal import Decimal
from django.core.urlresolvers import reverse
from django.db.models.loading import get_model

from django.test import TestCase
from django.test.client import Client
import mock
from getpaid.backends import przelewy24
import getpaid.backends.payu
import getpaid.backends.transferuj

from getpaid_test_project.orders.models import Order


class TransferujBackendTestCase(TestCase):
    def test_online_not_allowed_ip(self):
        self.assertEqual('IP ERR',
                         getpaid.backends.transferuj.PaymentProcessor.online('0.0.0.0', None, None, None, None, None,
                                                                             None, None, None, None, None, None))

        #Tests allowing IP given in settings
        with self.settings(GETPAID_BACKENDS_SETTINGS={
            'getpaid.backends.transferuj': {'allowed_ip': ('1.1.1.1', '1.2.3.4'), 'key': ''},
        }):
            self.assertEqual('IP ERR',
                             getpaid.backends.transferuj.PaymentProcessor.online('0.0.0.0', None, None, None, None,
                                                                                 None, None, None, None, None, None,
                                                                                 None))
            self.assertNotEqual('IP ERR',
                                getpaid.backends.transferuj.PaymentProcessor.online('1.1.1.1', None, None, None, None,
                                                                                    None, None, None, None, None, None,
                                                                                    None))
            self.assertNotEqual('IP ERR',
                                getpaid.backends.transferuj.PaymentProcessor.online('1.2.3.4', None, None, None, None,
                                                                                    None, None, None, None, None, None,
                                                                                    None))

        #Tests allowing all IP
        with self.settings(GETPAID_BACKENDS_SETTINGS={
            'getpaid.backends.transferuj': {'allowed_ip': [], 'key': ''},
        }):
            self.assertNotEqual('IP ERR',
                                getpaid.backends.transferuj.PaymentProcessor.online('0.0.0.0', None, None, None, None,
                                                                                    None, None, None, None, None, None,
                                                                                    None))
            self.assertNotEqual('IP ERR',
                                getpaid.backends.transferuj.PaymentProcessor.online('1.1.1.1', None, None, None, None,
                                                                                    None, None, None, None, None, None,
                                                                                    None))
            self.assertNotEqual('IP ERR',
                                getpaid.backends.transferuj.PaymentProcessor.online('1.2.3.4', None, None, None, None,
                                                                                    None, None, None, None, None, None,
                                                                                    None))

    def test_online_wrong_sig(self):
        self.assertEqual('SIG ERR',
                         getpaid.backends.transferuj.PaymentProcessor.online('195.149.229.109', '1234', '1', '', '1',
                                                                             '123.45', None, None, None, None, None,
                                                                             'xxx'))
        self.assertNotEqual('SIG ERR',
                            getpaid.backends.transferuj.PaymentProcessor.online('195.149.229.109', '1234', '1', '', '1',
                                                                                '123.45', None, None, None, None, None,
                                                                                '21b028c2dbdcb9ca272d1cc67ed0574e'))

    def test_online_wrong_id(self):
        self.assertEqual('ID ERR',
                         getpaid.backends.transferuj.PaymentProcessor.online('195.149.229.109', '1111', '1', '', '1',
                                                                             '123.45', None, None, None, None, None,
                                                                             '15bb75707d4374bc6e578c0cbf5a7fc7'))
        self.assertNotEqual('ID ERR',
                            getpaid.backends.transferuj.PaymentProcessor.online('195.149.229.109', '1234', '1', '', '1',
                                                                                '123.45', None, None, None, None, None,
                                                                                'f5f8276fbaa98a6e05b1056ab7c3a589'))

    def test_online_crc_error(self):
        self.assertEqual('CRC ERR',
                         getpaid.backends.transferuj.PaymentProcessor.online('195.149.229.109', '1234', '1', '',
                                                                             '99999', '123.45', None, None, None, None,
                                                                             None, 'f5f8276fbaa98a6e05b1056ab7c3a589'))
        self.assertEqual('CRC ERR',
                         getpaid.backends.transferuj.PaymentProcessor.online('195.149.229.109', '1234', '1', '',
                                                                             'GRRGRRG', '123.45', None, None, None,
                                                                             None, None,
                                                                             '6a9e045010c27dfed24774b0afa37d0b'))

    def test_online_payment_ok(self):
        Payment = get_model('getpaid', 'Payment')
        order = Order(name='Test EUR order', total='123.45', currency='PLN')
        order.save()
        payment = Payment(order=order, amount=order.total, currency=order.currency, backend='getpaid.backends.payu')
        payment.save(force_insert=True)
        self.assertEqual('TRUE', getpaid.backends.transferuj.PaymentProcessor.online('195.149.229.109', '1234', '1', '',
                                                                                     payment.pk, '123.45', '123.45', '',
                                                                                     'TRUE', 0, '',
                                                                                     '21b028c2dbdcb9ca272d1cc67ed0574e'))
        payment = Payment.objects.get(pk=payment.pk)
        self.assertEqual(payment.status, 'paid')
        self.assertNotEqual(payment.paid_on, None)
        self.assertEqual(payment.amount_paid, Decimal('123.45'))

    def test_online_payment_ok_over(self):
        Payment = get_model('getpaid', 'Payment')
        order = Order(name='Test EUR order', total='123.45', currency='PLN')
        order.save()
        payment = Payment(order=order, amount=order.total, currency=order.currency, backend='getpaid.backends.payu')
        payment.save(force_insert=True)
        self.assertEqual('TRUE', getpaid.backends.transferuj.PaymentProcessor.online('195.149.229.109', '1234', '1', '',
                                                                                     payment.pk, '123.45', '223.45', '',
                                                                                     'TRUE', 0, '',
                                                                                     '21b028c2dbdcb9ca272d1cc67ed0574e'))
        payment = Payment.objects.get(pk=payment.pk)
        self.assertEqual(payment.status, 'paid')
        self.assertNotEqual(payment.paid_on, None)
        self.assertEqual(payment.amount_paid, Decimal('223.45'))

    def test_online_payment_partial(self):
        Payment = get_model('getpaid', 'Payment')
        order = Order(name='Test EUR order', total='123.45', currency='PLN')
        order.save()
        payment = Payment(order=order, amount=order.total, currency=order.currency, backend='getpaid.backends.payu')
        payment.save(force_insert=True)
        self.assertEqual('TRUE', getpaid.backends.transferuj.PaymentProcessor.online('195.149.229.109', '1234', '1', '',
                                                                                     payment.pk, '123.45', '23.45', '',
                                                                                     'TRUE', 0, '',
                                                                                     '21b028c2dbdcb9ca272d1cc67ed0574e'))
        payment = Payment.objects.get(pk=payment.pk)
        self.assertEqual(payment.status, 'partially_paid')
        self.assertNotEqual(payment.paid_on, None)
        self.assertEqual(payment.amount_paid, Decimal('23.45'))

    def test_online_payment_failure(self):
        Payment = get_model('getpaid', 'Payment')
        order = Order(name='Test EUR order', total='123.45', currency='PLN')
        order.save()
        payment = Payment(order=order, amount=order.total, currency=order.currency, backend='getpaid.backends.payu')
        payment.save(force_insert=True)
        self.assertEqual('TRUE', getpaid.backends.transferuj.PaymentProcessor.online('195.149.229.109', '1234', '1', '',
                                                                                     payment.pk, '123.45', '23.45', '',
                                                                                     False, 0, '',
                                                                                     '21b028c2dbdcb9ca272d1cc67ed0574e'))
        payment = Payment.objects.get(pk=payment.pk)
        self.assertEqual(payment.status, 'failed')


def fake_payment_get_response_success(request):
    class fake_response:
        def read(self):
            return """
status:OK
trans_id:234748067
trans_pos_id:123456789
trans_session_id:99:1342616247.41
trans_order_id:99
trans_amount:12345
trans_status:99
trans_pay_type:t
trans_pay_gw_name:pt
trans_desc:Test 2
trans_desc2:
trans_create:2012-07-18 14:57:28
trans_init:
trans_sent:
trans_recv:
trans_cancel:20trans_12-07-18 14:57:30
trans_auth_fraud:0
trans_ts:1342616255805
trans_sig:4d4df5557b89a4e2d8c48436b1dd3fef
"""

    return fake_response()


def fake_payment_get_response_failure(request):
    class fake_response:
        def read(self):
            return """
status:OK
trans_id:234748067
trans_pos_id:123456789
trans_session_id:98:1342616247.41
trans_order_id:98
trans_amount:12345
trans_status:2
trans_pay_type:t
trans_pay_gw_name:pt
trans_desc:Test 2
trans_desc2:
trans_create:2012-07-18 14:57:28
trans_init:
trans_sent:
trans_recv:
trans_cancel:2012-07-18 14:57:30
trans_auth_fraud:0
trans_ts:1342616255805
trans_sig:ee77e9515599e3fd2b3721dff50111dd
"""

    return fake_response()


class PayUBackendTestCase(TestCase):
    def setUp(self):
        self.client = Client()


    def test_parse_text_result(self):
        t1 = '''status:OK

trans_id:349659572
trans_pos_id:105664
trans_session_id:48:1379695300.48
trans_ts:1379695309225
trans_sig:e4e981bfa780fa78fb077ca1f9295f2a

        '''
        self.assertEqual(getpaid.backends.payu.PaymentProcessor._parse_text_response(t1),
                         {
                             'status': 'OK',
                             'trans_id': '349659572',
                             'trans_pos_id': '105664',
                             'trans_session_id': '48:1379695300.48',
                             'trans_ts': '1379695309225',
                             'trans_sig': 'e4e981bfa780fa78fb077ca1f9295f2a',
                         }
        )

    def test_online_malformed(self):
        response = self.client.post(reverse('getpaid-payu-online'), {})
        self.assertEqual(response.content, 'MALFORMED')

    def test_online_sig_err(self):
        response = self.client.post(reverse('getpaid-payu-online'), {
            'pos_id': 'wrong',
            'session_id': '10:11111',
            'ts': '1111',
            'sig': 'wrong sig',
        })
        self.assertEqual(response.content, 'SIG ERR')

    def test_online_wrong_pos_id_err(self):
        response = self.client.post(reverse('getpaid-payu-online'), {
            'pos_id': '12345',
            'session_id': '10:11111',
            'ts': '1111',
            'sig': '0d6129738c0aee9d4eb56f2a1db75ab4',
        })
        self.assertEqual(response.content, 'POS_ID ERR')

    def test_online_wrong_session_id_err(self):
        response = self.client.post(reverse('getpaid-payu-online'), {
            'pos_id': '123456789',
            'session_id': '111111',
            'ts': '1111',
            'sig': 'fcf3db081d5085b45fe86ed0c6a9aa5e',
        })
        self.assertEqual(response.content, 'SESSION_ID ERR')

    def test_online_ok(self):
        response = self.client.post(reverse('getpaid-payu-online'), {
            'pos_id': '123456789',
            'session_id': '1:11111',
            'ts': '1111',
            'sig': '2a78322c06522613cbd7447983570188',
        })
        self.assertEqual(response.content, 'OK')

    @mock.patch("urllib2.urlopen", fake_payment_get_response_success)
    def test_payment_get_paid(self):
        Payment = get_model('getpaid', 'Payment')
        order = Order(name='Test EUR order', total='123.45', currency='PLN')
        order.save()
        payment = Payment(pk=99, order=order, amount=order.total, currency=order.currency,
                          backend='getpaid.backends.payu')
        payment.save(force_insert=True)
        payment = Payment.objects.get(
            pk=99)  # this line is because django bug https://code.djangoproject.com/ticket/5903
        processor = getpaid.backends.payu.PaymentProcessor(payment)
        processor.get_payment_status('99:1342616247.41')
        self.assertEqual(payment.status, 'paid')
        self.assertNotEqual(payment.paid_on, None)
        self.assertNotEqual(payment.amount_paid, Decimal('0'))

    @mock.patch("urllib2.urlopen", fake_payment_get_response_failure)
    def test_payment_get_failed(self):
        Payment = get_model('getpaid', 'Payment')
        order = Order(name='Test EUR order', total='123.45', currency='PLN')
        order.save()
        payment = Payment(pk=98, order=order, amount=order.total, currency=order.currency,
                          backend='getpaid.backends.payu')
        payment.save(force_insert=True)
        payment = Payment.objects.get(
            pk=98)  # this line is because django bug https://code.djangoproject.com/ticket/5903
        processor = getpaid.backends.payu.PaymentProcessor(payment)
        processor.get_payment_status('98:1342616247.41')
        self.assertEqual(payment.status, 'failed')
        self.assertEqual(payment.paid_on, None)
        self.assertEqual(payment.amount_paid, Decimal('0'))


def fake_przelewy24_payment_get_response_success(request):
    class fake_response:
        def read(self):
            return """RESULT
TRUE"""

    return fake_response()


def fake_przelewy24_payment_get_response_failed(request):
    class fake_response:
        def read(self):
            return """RESULT
ERR
123
Some error description"""

    return fake_response()


class Przelewy24PaymentProcessorTestCase(TestCase):
    def test_sig(self):
        # Test based on p24 documentation
        sig = przelewy24.PaymentProcessor.compute_sig({
                                                          'key1': '9999',
                                                          'key2': '2500',
                                                          'key3': 'ccc',
                                                          'key4': 'abcdefghijk',
                                                          'crc': 'a123b456c789d012',

                                                      }, ('key4', 'key1', 'key2', 'crc'), 'a123b456c789d012')
        self.assertEqual(sig, 'e2c43dec9578633c518e1f514d3b434b')

    @mock.patch("urllib2.urlopen", fake_przelewy24_payment_get_response_success)
    def test_get_payment_status_success(self):
        Payment = get_model('getpaid', 'Payment')
        order = Order(name='Test PLN order', total='123.45', currency='PLN')
        order.save()
        payment = Payment(pk=191, order=order, amount=order.total, currency=order.currency,
                          backend='getpaid.backends.przelewy24')
        payment.save(force_insert=True)
        payment = Payment.objects.get(pk=191)
        processor = getpaid.backends.przelewy24.PaymentProcessor(payment)
        processor.get_payment_status(p24_session_id='191:xxx:xxx', p24_order_id='191:external', p24_kwota='12345')
        self.assertEqual(payment.status, 'paid')
        self.assertEqual(payment.external_id, '191:external')
        self.assertNotEqual(payment.paid_on, None)
        self.assertEqual(payment.amount_paid, Decimal('123.45'))

    @mock.patch("urllib2.urlopen", fake_przelewy24_payment_get_response_success)
    def test_get_payment_status_success_partial(self):
        Payment = get_model('getpaid', 'Payment')
        order = Order(name='Test PLN order', total='123.45', currency='PLN')
        order.save()

        payment = Payment(pk=192, order=order, amount=order.total, currency=order.currency,
                          backend='getpaid.backends.przelewy24')
        payment.save(force_insert=True)
        payment = Payment.objects.get(pk=192)
        processor = getpaid.backends.przelewy24.PaymentProcessor(payment)
        processor.get_payment_status(p24_session_id='192:xxx:xxx', p24_order_id='192:external', p24_kwota='12245')
        self.assertEqual(payment.status, 'partially_paid')
        self.assertEqual(payment.external_id, '192:external')
        self.assertNotEqual(payment.paid_on, None)
        self.assertEqual(payment.amount_paid, Decimal('122.45'))

    @mock.patch("urllib2.urlopen", fake_przelewy24_payment_get_response_failed)
    def test_get_payment_status_failed(self):
        Payment = get_model('getpaid', 'Payment')
        order = Order(name='Test PLN order', total='123.45', currency='PLN')
        order.save()

        payment = Payment(pk=192, order=order, amount=order.total, currency=order.currency,
                          backend='getpaid.backends.przelewy24')
        payment.save(force_insert=True)
        payment = Payment.objects.get(pk=192)
        processor = getpaid.backends.przelewy24.PaymentProcessor(payment)
        processor.get_payment_status(p24_session_id='192:xxx:xxx', p24_order_id='192:external', p24_kwota='12245')
        self.assertEqual(payment.status, 'failed')
        self.assertEqual(payment.paid_on, None)
        self.assertEqual(payment.amount_paid, Decimal('0.0'))




# -*- coding: utf-8 -*-
from decimal import Decimal
import logging
import datetime
from django.core.urlresolvers import reverse
from django.db.models import get_model
from django.utils.timezone import utc
import requests
import time
from getpaid.signals import user_data_query
from getpaid.backends import PaymentProcessorBase
from lxml import etree

logger = logging.getLogger('getpaid.backends.moip')


class MoipTransactionStatus:
    AUTHORIZED = 1
    STARTED = 2
    IN_PROGRESS = 3
    AVAILABLE = 4
    CANCELED = 5
    PENDING = 6
    CHARGEBACK = 7
    IN_DISPUTE = 8
    REFUNDED = 9


class PaymentProcessor(PaymentProcessorBase):
    BACKEND = 'getpaid.backends.moip'
    BACKEND_NAME = 'Moip'
    BACKEND_ACCEPTED_CURRENCY = ('BRL', )

    _SEND_INSTRUCTION_PAGE = '/ws/alpha/EnviarInstrucao/Unica'
    _RUN_INSTRUCTION_PAGE = 'Instrucao.do?token='
    _ITEM_DATA_REQUIRED_FIELDS = ('id', 'description', 'quantity', 'value')

    _USER_DATA_TO_MOIP = {
        'name': 'Nome',  # Nome completo do cliente.
        'address': 'Logradouro',  # Logradouro do cliente (ex: Rua, Av, etc.)
        'address_number': 'Numero',  # Nº do imóvel do cliente (ex: 12)
        'address_complement': 'Complemento',  # Complemento (ex: Sala 109 ou Casa 1)
        'address_zip_code': 'CEP',  # O CEP de 8 dígitos do cliente. Somente números (ex: 22345678)
        'address_quarter': 'Bairro',  # Bairro do cliente
        'address_city': 'Cidade',  # Cidade do cliente (ex: São Paulo)
        'address_state': 'Estado',  # Estado do cliente (SP) no formato duas letras
        'phone': 'TelefoneFixo',  # Telefone fixo do cliente
        'email': 'Email',  # E-mail do cliente
    }

    def get_gateway_url(self, request):
        if PaymentProcessor.get_backend_setting('testing', False):
            gateway_url = "https://desenvolvedor.moip.com.br/sandbox"
        else:
            gateway_url = "https://www.moip.com.br"

        xml_body = etree.Element("EnviarInstrucao")
        xml_instruction = etree.SubElement(xml_body, "InstrucaoUnica")

        etree.SubElement(xml_instruction, "Razao").text = self.get_order_description(self.payment, self.payment.order)

        xml_values = etree.SubElement(xml_instruction, "Valores")
        etree.SubElement(xml_values, "Valor", moeda=self.payment.currency).text = str(self.payment.amount)

        etree.SubElement(xml_instruction, "IdProprio").text = "%s-%s" % (str(self.payment.id), str(time.time()))
        etree.SubElement(xml_instruction, "URLRetorno").text = PaymentProcessor._get_view_full_url(request, 'getpaid-moip-success', args=(self.payment.id,))
        etree.SubElement(xml_instruction, "URLNotificacao").text = PaymentProcessor._get_view_full_url(request, 'getpaid-moip-notifications')

        # collect customer data
        customer_info = {}
        user_data_query.send(sender=None, order=self.payment.order, user_data=customer_info)

        if customer_info:
            xml_buyer = etree.SubElement(xml_instruction, "Pagador")

            for field in ('name', 'email'):
                if field in customer_info and field in ('name', 'email'):
                    etree.SubElement(xml_buyer, self._USER_DATA_TO_MOIP[field]).text = customer_info[field]

            xml_buyer_address = etree.SubElement(xml_buyer, "EnderecoCobranca")
            for field in self._USER_DATA_TO_MOIP:
                if field in customer_info and field not in ('name', 'email'):
                    etree.SubElement(xml_buyer_address, self._USER_DATA_TO_MOIP[field]).text = customer_info[field]

        payment_full_url = "%s%s" % (gateway_url, self._SEND_INSTRUCTION_PAGE)
        user = PaymentProcessor.get_backend_setting('token')
        pwd = PaymentProcessor.get_backend_setting('key')
        contents = etree.tostring(xml_body, encoding='utf-8')

        response = requests.post(payment_full_url, auth=(user, pwd), data=contents).text
        moip_payment_token = etree.XML(response)[0][2].text

        return "%s/%s%s " % (gateway_url, self._RUN_INSTRUCTION_PAGE, moip_payment_token), 'GET', {}

    @staticmethod
    def process_notification(params):
        Payment = get_model('getpaid', 'Payment')
        try:
            payment = Payment.objects.get(pk=int(params["id"].split("-")[0]))
        except Payment.DoesNotExist:
            logger.error('Payment does not exist with pk=%d' % params["id"])
            return

        status_code = int(params["status"])
        if status_code in (MoipTransactionStatus.AUTHORIZED,
                           MoipTransactionStatus.AVAILABLE):
            payment.amount_paid = Decimal(params["amount"])
            payment.paid_on = datetime.datetime.utcnow().replace(tzinfo=utc)
            payment.change_status('paid')
        elif status_code in (MoipTransactionStatus.CANCELED,
                             MoipTransactionStatus.REFUNDED,
                             MoipTransactionStatus.CHARGEBACK):
            payment.change_status('failed')

    @staticmethod
    def _get_view_full_url(request, view_name, args=None):
        url = reverse(view_name, args=args)
        return 'http://%s%s' % (request.get_host(), url)



from django.contrib.sites.models import Site
from django.core.management.base import BaseCommand
from django.core.urlresolvers import reverse
from getpaid.backends.przelewy24 import PaymentProcessor


class Command(BaseCommand):
    help = 'Additional Przelewy24 configuration'

    def handle(self, *args, **options):
        current_site = Site.objects.get_current()

        self.stdout.write(
            'Please contact with Przelewy24 (serwis@przelewy24.pl) and provide them with the following URL: \n\n')

        self.stdout.write(
            ('https://' if PaymentProcessor.get_backend_setting('ssl_return', False) else 'http://') + '%s%s\n\n' % (
                current_site.domain, reverse('getpaid-przelewy24-online'))
        )

        self.stdout.write(
            'This is an additional URL for accepting payment status updates.\n\n')

        self.stdout.write(
            'To change domain name please edit Sites settings. Don\'t forget to setup your web server to accept https connection in order to use secure links.\n')
        if PaymentProcessor.get_backend_setting('sandbox', False):
            self.stdout.write('\nSandbox mode is ON.\n')



from django.conf.urls import patterns, url
from django.views.decorators.csrf import csrf_exempt
from getpaid.backends.moip.views import NotificationsView, SuccessView

urlpatterns = patterns('',
    url(r'^notifications/$', csrf_exempt(NotificationsView.as_view()), name='getpaid-moip-notifications'),
    url(r'^success/(?P<pk>\d+)/$', csrf_exempt(SuccessView.as_view()), name='getpaid-moip-success'),
)



import logging
from django.core.urlresolvers import reverse
from django.http import HttpResponse, HttpResponseRedirect
from django.views.generic.base import View
from django.views.generic.detail import DetailView
from getpaid.backends.przelewy24 import PaymentProcessor
from getpaid.models import Payment

logger = logging.getLogger('getpaid.backends.przelewy24')


class OnlineView(View):
    """
    This View answers on Przelewy24 online request that is acknowledge of payment
    status change.
    """

    def post(self, request, *args, **kwargs):
        try:
            p24_session_id = request.POST['p24_session_id']
            p24_order_id = request.POST['p24_order_id']
            p24_kwota = request.POST['p24_kwota']
            p24_order_id_full = request.POST['p24_order_id_full']
            p24_crc = request.POST['p24_crc']
        except KeyError:
            logger.warning('Got malformed POST request: %s' % str(request.POST))
            return HttpResponse('MALFORMED', status=500)

        if PaymentProcessor.on_payment_status_change(p24_session_id, p24_order_id, p24_kwota, p24_order_id_full,
                                                     p24_crc):
            return HttpResponse('OK')
        else:
            return HttpResponse('CRC ERR')


class SuccessView(DetailView):
    """
    This view just redirects to standard backend success link after it schedule payment status checking.
    """
    model = Payment

    def get(self, request, *args, **kwargs):
        return HttpResponse('GET not allowed')

    def post(self, request, *args, **kwargs):
        try:
            p24_session_id = request.POST['p24_session_id']
            p24_order_id = request.POST['p24_order_id']
            p24_kwota = request.POST['p24_kwota']
            p24_order_id_full = request.POST['p24_order_id_full']
            p24_crc = request.POST['p24_crc']
        except KeyError:
            logger.warning('Got malformed POST request: %s' % str(request.POST))
            return HttpResponse('MALFORMED', status=500)

        PaymentProcessor.on_payment_status_change(p24_session_id, p24_order_id, p24_kwota, p24_order_id_full, p24_crc)

        self.object = self.get_object()
        context = self.get_context_data(object=self.object)
        return self.render_to_response(context)

    def render_to_response(self, context, **response_kwargs):
        return HttpResponseRedirect(reverse('getpaid-success-fallback', kwargs={'pk': self.object.pk}))


class FailureView(DetailView):
    """
    This view just redirects to standard backend failure link after it schedule payment status checking.
    """
    model = Payment

    def post(self, request, *args, **kwargs):
        try:
            p24_session_id = request.POST['p24_session_id']
            p24_order_id = request.POST['p24_order_id']
            p24_kwota = request.POST['p24_kwota']
            p24_order_id_full = request.POST['p24_order_id_full']
            p24_crc = request.POST['p24_crc']
        except KeyError:
            logger.warning('Got malformed POST request: %s' % str(request.POST))
            return HttpResponse('MALFORMED', status=500)

        PaymentProcessor.on_payment_status_change(p24_session_id, p24_order_id, p24_kwota, p24_order_id_full, p24_crc)

        self.object = self.get_object()
        context = self.get_context_data(object=self.object)
        return self.render_to_response(context)

    def render_to_response(self, context, **response_kwargs):
        logger.error("Payment %s failed on backend error" % (self.kwargs['pk'],))
        return HttpResponseRedirect(reverse('getpaid-failure-fallback', kwargs={'pk': self.object.pk}))



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#pymill.py
# From https://github.com/kliment/pymill since it's not on PyPi

import requests


class Pymill():
    """
    These are the parameters each object type contains

    Payment:
    id: unique payment method ID
    type: creditcard or debit
    client: id of associatied client (optional)
    created_at: unixtime
    updated_at: unixtime
    (For credit cards only)
    card_type: visa, mastercard, (maybe one day american express)
    country: country the card was issued in
    expire_month: (2ch)
    expire_year: (4ch)
    card_holder: name of cardholder
    last4: last 4 digits of card
    (For debit accounts only)
    code: the sorting code of the bank
    account: a partially masked account number
    holder: name of the account holder

    Preauthorization:
    id: unique preauthorization ID
    amount: amount preauthorized in CENTS
    status: open, pending, closed, failed, deleted, or preauth
    livemode: true or false depending on whether the transaction is real or in test mode
    payment: a credit card payment method object (see above)
    client: if a preset client (see below) was used to make the transaction. Otherwise null
    created_at: unixtime
    updated_at: unixtime

    Transaction:
    id: unique transaction ID
    amount: amount charged in CENTS
    status: open, pending, closed, failed, partial_refunded, refunded, or preauthorize (closed means success)
    description: user-selected description of the transaction
    livemode: true or false depending on whether the transaction is real or in test mode
    payment: a payment method object (see above)
    preauthorization: the preauthorization associated with this transaction (optional)
    created_at: unixtime
    updated_at: unixtime

    Refund:
    id: unique refund ID
    transaction: The unique transaction ID of the transaction being refunded
    amount: amount refunded in CENTS
    status: open, pending or refunded
    description: user-selected description of the refund
    livemode: true or false depending on whether the transaction is real or in test mode
    created_at: unixtime
    updated_at: unixtime

    Client:
    id: unique id for this client
    email: client's email address (optional)
    description: description of this client (optional)
    created_at: unix timestamp identifying time of creation
    updated_at: unix timestamp identifying time of last change
    payment: list of cc or debit objects
    subscription: subscription object (optional)

    Offer:
    id: unique offer identifier
    name: freely controllable offer name
    amount: The amount, in CENTS, to be charged every time the offer period passes. Note that ODD values will NOT work in test mode.
    interval: "week", "month", or "year". The client will be charged every time the interval passes
    trial_period_days: Number of days before the first charge. (optional)

    Subscription:
    id: unique subscription identifier
    offer: unique offer identifier
    livemode: true or false depending on whether the transaction is real or in test mode
    cancel_at_period_end: true if subscription is to be cancelled at the end of current period, false if to be cancelled immediately
    created_at: unix timestamp identifying time of creation
    updated_at: unix timestamp identifying time of last change
    canceled_at: unix timestamp identifying time of cancellation(optional)
    interval: "week", "month", or "year". The client will be charged every time the interval passes
    clients: array of client objects


    """

    def __init__(self, privatekey):
        """
        Initialize a new paymill interface connection. Requires a private key.
        """
        self.s = requests.Session()
        self.s.auth = (privatekey, "")
        self.s.verify = False
        #self.s.cert="cacert.pem"

    def _apicall(self, url, params=(), cr="GET", ch=None):
        """
        Call an API endpoint.
        url: The URL of the entity to post to.
        params: a tuple of (name, value) tuples of parameters
        cr: The request type to be made. If parameters are passed, this will be ignored and treated as POST

        Returns a dictionary object populated with the json returned.
        """
        pars = dict(params)
        rf = None
        if(cr == "GET"):
            rf = self.s.get
        elif cr == "DELETE":
            rf = self.s.delete
        elif cr == "PUT":
            rf = self.s.put
        else:
            rf = self.s.post
        if params is not () and cr != "DELETE"and cr != "PUT":
            rf = self.s.post
        r = None
        if(ch):
            r = rf(url, params=pars, headers=ch)
        else:
            r = rf(url, params=pars)
        if ch is not None:
            return r.text
        return r.json()

    def newdebit(self, code, account, holder, client=None):
        """
        Create a debit card from account data.
        code: A bank sorting code
        account: The account number
        holder: The name of the account holder
        client: A client id number (optional)


        Returns: a dict with a member "data" containing a dict representing a debit card
        """
        p = []
        if code is not None and account is not None and holder is not None:
            p += [("code", code), (
                "account", account), ("holder", holder), ("type", "debit")]
        if client is not None:
            p += [("client", client)]
        return self._apicall("https://api.paymill.de/v2/payments/", tuple(p))

    def newcard(self, token, client=None):
        """
        Create a credit card from a given token.
        token: string Unique credit card token
        client: A client id number (optional)

        Returns: a dict with a member "data" containing a dict representing a CC
        """
        p = []
        if client is not None:
            p += [("client", client)]
        p += [("token", token)]
        return self._apicall("https://api.paymill.de/v2/payments", tuple(p))

    def getcarddetails(self, cardid):
        """
        Get the details of a credit card from its id.
        cardid: string Unique id for the credit card

        Returns: a dict with a member "data" containing a dict representing a CC
        """
        return self._apicall("https://api.paymill.de/v2/payments/" + str(cardid))

    def getcards(self):
        """
        List all stored cards.

        Returns: a dict with a member "data" which is an array of dicts, each representing a CC or debit card
        """
        return self._apicall("https://api.paymill.de/v2/payments/")

    def delcard(self, cardid):
        """
        Delete a stored CC
        cardid: Unique id for the CC to be deleted

        Returns: a dict with an member "data" containing an empty array
        """
        return self._apicall("https://api.paymill.de/v2/payments/%s" % (str(cardid),), cr="DELETE")

    def transact(self, amount=0, currency="eur", description=None, token=None, client=None, payment=None, preauth=None, code=None, account=None, holder=None):
        """
        Create a transaction (charge a card or account). You must provide an amount, and exactly one funding source.
        The amount is in cents, and the funding source can be a payment method id, a token, a preauthorization or a direct debit account.
        amount: The amount (in CENTS) to be charged. For example, 240 will charge 2 euros and 40 cents, NOT 240 euros.
        currency: ISO4217 currency code (optional)
        description: A short description of the transaction (optional)
        token: A token generated by the paymill bridge js library
        client: A client id number (optional)
        payment: A payment method id number (credit card id or debit account id)
        preauth: A preauthorization id number
        code: If paying by debit, the bank sorting code
        account: If paying by debit, the account number
        holder: If paying by debit, the name of the account holder

        Returns: None if one of the required parameters is missing. A dict with a member "data" containing a transaction dict otherwise.
        """
        p = []
        if client is not None:
            p += [("client", client)]
        if payment is None and code is not None and account is not None and holder is not None:
            try:
                payment = self.newdebit(
                    code, account, holder, client)["data"]["id"]
            except:
                return self.newdebit(code, account, holder, client)
        if payment is not None:
            p += [("payment", payment)]
        elif token is not None:
            p += [("token", token)]
        elif preauth is not None:
            p += [("preauthorization", preauth)]
        else:
            return None
        if amount == 0:
            return None
        if description is not None:
            p += [("description", description)]
        p += [("amount", str(amount))]
        p += [("currency", currency)]
        return self._apicall("https://api.paymill.de/v2/transactions/", tuple(p))

    def gettransdetails(self, tranid):
        """
        Get details on a transaction.
        tranid: string Unique id for the transaction

        Returns: a dict representing a transaction
        """
        return self._apicall("https://api.paymill.de/v2/transactions/" + str(tranid))

    def gettrans(self):
        """
        List all transactions.

        Returns: a dict with a member "data" which is an array of dicts, each representing a transaction
        """
        return self._apicall("https://api.paymill.de/v2/transactions/")

    def refund(self, tranid, amount, description=None):
        """
        Refunds an already performed transaction.
        tranid: string Unique transaction id
        amount: The amount in cents that are to be refunded
        description: A description of the refund (optional)

        Returns: a dict with a member "data" which is a dict representing a refund, or None if the amount is 0
        """
        if amount == 0:
            return None
        p = [("amount", str(amount))]
        if description is not None:
            p += [("description", description)]
        return self._apicall("https://api.paymill.de/v2/refunds/" + str(tranid), tuple(p))

    def getrefdetails(self, refid):
        """
        Get the details of a refund from its id.
        refid: string Unique id for the refund

        Returns: a dict with a member "data" which is a dict representing a refund
        """
        return self._apicall("https://api.paymill.de/v2/refunds/" + str(refid))

    def getrefs(self):
        """
        List all stored refunds.

        Returns: a dict with a member "data" which is an array of dicts, each representing a refund
        """
        return self._apicall("https://api.paymill.de/v2/refunds/")

    def preauth(self, amount=0, currency="eur", description=None, token=None, client=None, payment=None):
        """
        Preauthorize a transaction (reserve value a card). You must provide an amount, and exactly one funding source.
        The amount is in cents, and the funding source can be a token or a payment id.
        amount: The amount (in CENTS) to be charged. For example, 240 will charge 2 euros and 40 cents, NOT 240 euros.
        currency: ISO4217 (optional)
        token: A token generated by the paymill bridge js library
        payment: A payment method id number. Must represent a credit card, not a debit payment.

        Returns: None if one of the required parameters is missing. A dict with a member "data" containing a preauthorization dict otherwise.
        """
        p = []
        if payment is not None:
            p += [("payment", payment)]
        elif token is not None:
            p += [("token", token)]
        else:
            return None
        if amount == 0:
            return None
        p += [("amount", str(amount))]
        p += [("currency", currency)]
        return self._apicall("https://api.paymill.de/v2/preauthorizations/", tuple(p))

    def getpreauthdetails(self, preid):
        """
        Get details on a preauthorization.
        preid: string Unique id for the preauthorization

        Returns: a dict representing a preauthorization
        """
        return self._apicall("https://api.paymill.de/v2/preauthorizations/" + str(preid))

    def getpreauth(self):
        """
        List all preauthorizations.

        Returns: a dict with a member "data" which is an array of dicts, each representing a preauthorization
        """
        return self._apicall("https://api.paymill.de/v2/preauthorizations/")

    def newclient(self, email=None, description=None):
        """
        Creates a new client.
        email: client's email address
        description: description of this client (optional)

        Returns: a dict with a member "data" which is a dict representing a client.
        """
        p = []
        if description is not None:
            p += [("description", description)]
        if email is not None:
            p += [("email", str(email))]
        if p is []:
            return None
        return self._apicall("https://api.paymill.de/v2/clients", tuple(p))

    def getclientdetails(self, cid):
        """
        Get the details of a client from its id.
        cid: string Unique id for the client

        Returns: a dict with a member "data" which is a dict representing a client
        """
        return self._apicall("https://api.paymill.de/v2/clients/" + str(cid))

    def updateclient(self, cid, email, description=None):
        """
        Updates the details of a client.
        cid: string Unique client id
        email: The email of the client
        description: A description of the client (optional)

        Returns: a dict with a member "data" which is a dict representing a client
        """
        p = []
        if description is not None:
            p += [("description", description)]
        if email is not None:
            p += [("email", str(email))]
        if p is []:
            return None
        return self._apicall("https://api.paymill.de/v2/clients/" + str(cid), tuple(p), cr="PUT")

    def delclient(self, cid):
        """
        Delete a stored client
        cid: Unique id for the client to be deleted

        Returns: a dict with an member "data" containing an empty array
        """
        return self._apicall("https://api.paymill.de/v2/clients/%s" % (str(cid),), cr="DELETE")

    def getclients(self):
        """
        List all stored clients.

        Returns: a dict with a member "data" which is an array of dicts, each representing a client
        """
        return self._apicall("https://api.paymill.de/v2/clients/")

    def exportclients(self):
        """
        Export all stored clients in CSV form

        Returns: the contents of the CSV file
        """
        return self._apicall("https://api.paymill.de/v2/clients/", ch={"Accept": "text/csv"})

    def newoffer(self, amount, interval="month", currency="eur", name=None):
        """
        Creates a new offer
        amount: The amount in cents that are to be charged every interval
        interval: MUST be either "week", "month" or "year"
        currency: "eur" by default (optional)
        name: A name for this offer

        Returns: a dict with a member "data" which is a dict representing
            an offer, or None if the amount is 0 or the interval is invalid
        """
        if amount == 0:
            return None
        p = [("amount", str(amount))]
        if interval not in ["week", "month", "year"]:
            return None
        p += [("currency", str(currency))]
        p += [("interval", str(interval))]
        if name is not None:
            p += [("name", name)]
        return self._apicall("https://api.paymill.de/v2/offers", tuple(p))

    def getofferdetails(self, oid):
        """
        Get the details of an offer from its id.
        oid: string Unique id for the offer

        Returns: a dict with a member "data" which is a dict representing an offer
        """
        return self._apicall("https://api.paymill.de/v2/offers/" + str(oid))

    def updateoffer(self, oid, name):
        """
        Updates the details of an offer. Only the name may be changed
        oid: string Unique offer id
        name: The new name of the offer

        Returns: a dict with a member "data" which is a dict representing an offer
        """
        p = [("name", str(name))]
        return self._apicall("https://api.paymill.de/v2/offers/" + str(oid), tuple(p))

    def deloffer(self, oid):
        """
        Delete a stored offer. May only be done if no subscriptions to this offer are active.
        oid: Unique id for the offer to be deleted

        Returns: a dict with an member "data" containing an empty array
        """
        return self._apicall("https://api.paymill.de/v2/offers/%s" % (str(oid),), cr="DELETE")

    def getoffers(self):
        """
        List all stored offers.

        Returns: a dict with a member "data" which is an array of dicts, each representing an offer
        """
        return self._apicall("https://api.paymill.de/v2/offers/")

    def newsub(self, client, offer, payment):
        """
        Subscribes a client to an offer
        client: The id of the client
        offer: The id of the offer
        payment: The id of the payment instrument used for this offer

        Returns: a dict with a member "data" which is a dict representing a subscription
        """
        p = [("offer", str(offer)), ("client", str(client)), (
            "payment", str(payment))]
        return self._apicall("https://api.paymill.de/v2/subscriptions", tuple(p))

    def getsubdetails(self, sid):
        """
        Get the details of a subscription from its id.
        sid: string Unique id for the subscription

        Returns: a dict with a member "data" which is a dict representing a subscription
        """
        return self._apicall("https://api.paymill.de/v2/subscriptions/" + str(sid))

    def cancelsubafter(self, sid, cancel=True):
        """
        Cancels a subscription after its interval ends
        sid: string Unique subscription id
        cancel: If True, the subscription will be cancelled at the end of its interval. Set to False to undo.

        Returns: a dict with a member "data" which is a dict representing a subscription
        """
        if cancel:
            p = [("cancel_at_period_end", "true")]
        else:
            p = [("cancel_at_period_end", "false")]
        return self._apicall("https://api.paymill.de/v2/subscriptions/" + str(sid), tuple(p), cr="PUT")

    def cancelsubnow(self, sid):
        """
        Cancel a subscription immediately. Pending transactions will still be charged.
        sid: Unique subscription id

        Returns: a dict with an member "data"
        """
        return self._apicall("https://api.paymill.de/v2/subscriptions/%s" % (str(sid),), cr="DELETE")

    def getsubs(self):
        """
        List all stored subscriptions.

        Returns: a dict with a member "data" which is an array of dicts, each representing a subscription
        """
        return self._apicall("https://api.paymill.de/v2/subscriptions/")
        
    def response_code2text(self, response_code):
        texts = {
                10001:    'General undefined response.',
                10002:    'Still waiting on something.',

                20000:    'General success response.',
                
                40000:    'General problem with data.',
                40100:    'Problem with creditcard data.',
                40101:    'Problem with cvv.',
                40102:    'Card expired or not yet valid.',
                40103:    'Limit exceeded.',
                40104:    'Card invalid.',
                40105:    'expiry date not valid',
                40200:    'Problem with bank account data.',
                40300:    'Problem with 3d secure data.',
                40301:    'currency / amount mismatch',
                40400:    'Problem with input data.',
                40401:    'Amount too low or zero.',
                40402:    'Usage field too long.',
                40403:    'Currency not allowed.',
                
                50000:    'General problem with backend.',
                50001:    'country blacklisted.',
                50100:    'Technical error with credit card.',
                50101:    'Error limit exceeded.',
                50102:    'Card declined by authorization system.',
                50103:    'Manipulation or stolen card.',
                50104:    'Card restricted.',
                50105:    'Invalid card configuration data.',
                50200:    'Technical error with bank account.',
                50201:    'Card blacklisted.',
                50300:    'Technical error with 3D secure.',
                50400:    'Decline because of risk issues.',      
        }
        
        try:
            return texts[response_code]
        except:
            return response_code
    

if __name__ == "__main__":
    p = Pymill("YOURPRIVATEKEYHERE")
    cc = (p.getcards())["data"][0]["id"]
    print p.getcarddetails(cc)
    #print p.transact(amount=300,code="86055500",account="1234512345",holder="Max Mustermann",description="debittest")
    #print p.transact(amount=300,payment=cc,description="pymilltest")



import logging
from celery.task.base import task
from django.db.models.loading import get_model


logger = logging.getLogger('getpaid.backends.payu')


@task(max_retries=50, default_retry_delay=2*60)
def get_payment_status_task(payment_id, session_id):
    Payment = get_model('getpaid', 'Payment')
    try:
        payment = Payment.objects.get(pk=int(payment_id))
    except Payment.DoesNotExist:
        logger.error('Payment does not exist pk=%d' % payment_id)
        return
    from getpaid.backends.payu import PaymentProcessor # Avoiding circular import
    processor = PaymentProcessor(payment)
    processor.get_payment_status(session_id)


@task(max_retries=50, default_retry_delay=2 * 60)
def accept_payment(payment_id, session_id):
    Payment = get_model('getpaid', 'Payment')
    try:
        payment = Payment.objects.get(pk=int(payment_id))
    except Payment.DoesNotExist:
        logger.error('Payment does not exist pk=%d' % payment_id)
        return

    from getpaid.backends.payu import PaymentProcessor # Avoiding circular import
    processor = PaymentProcessor(payment)
    processor.accept_payment(session_id)


from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from getpaid.backends import PaymentProcessorBase

class PaymentProcessor(PaymentProcessorBase):
    BACKEND = 'getpaid.backends.paymill'
    BACKEND_NAME = _('Paymill')
    BACKEND_ACCEPTED_CURRENCY = ('EUR', 'CZK', 'DKK', 'HUF', 'ISK', 'ILS', 'LVL',
        'CHF', 'NOK', 'PLN', 'SEK', 'TRY', 'GBP', )

    def get_gateway_url(self, request):
        return reverse('getpaid-paymill-authorization', kwargs={'pk' : self.payment.pk}), "GET", {}


import logging
from django.core.urlresolvers import reverse
from django.http import HttpResponse, Http404, HttpResponseRedirect
from django.views.generic import DetailView
from django.views.generic.base import View
from getpaid.backends.moip import PaymentProcessor
from getpaid.models import Payment

logger = logging.getLogger('getpaid.backends.moip')


class NotificationsView(View):
    """
    This view answers on Moip notifications requests.
    See http://labs.moip.com.br/referencia/nasp/

    The most important logic of this view is delegated to the
    ``PaymentProcessor.processNotification()`` method
    """
    def post(self, request, *args, **kwargs):
        try:
            params = {
                'id': request.POST['id_transacao'],
                'amount': request.POST['valor'],
                'status': request.POST['status_pagamento'],
                'moip_id': request.POST['cod_moip'],
                'email': request.POST['email_consumidor'],
            }
        except KeyError:
            logger.warning('Got malformed POST request: %s' % str(request.POST))
            raise Http404

        status = PaymentProcessor.process_notification(params)
        return HttpResponse(status)


class SuccessView(DetailView):
    """
    This view just redirects to standard backend success link.
    """
    model = Payment

    def render_to_response(self, context, **response_kwargs):
        return HttpResponseRedirect(reverse('getpaid-success-fallback', kwargs={'pk': self.object.pk}))



import logging
from django.core.urlresolvers import reverse
from django.http import HttpResponse, HttpResponseRedirect
from django.views.generic.base import View
from django.views.generic.detail import DetailView
from getpaid.backends.dotpay import PaymentProcessor
from getpaid.models import Payment

logger = logging.getLogger('getpaid.backends.dotpay')


class OnlineView(View):
    """
    This View answers on PayU online request that is acknowledge of payment
    status change.

    The most important logic of this view is delegated to ``PaymentProcessor.online()`` method
    """
    def post(self, request, *args, **kwargs):

        try:
            params = {
                'id': request.POST['id'],
                'status': request.POST['status'],
                'control': request.POST['control'],
                't_id': request.POST['t_id'],
                'amount': request.POST['amount'],
                'email': request.POST['email'],

                'orginal_amount': request.POST['orginal_amount'],
                't_status': request.POST['t_status'],
                'md5': request.POST['md5'],


                'service': request.POST.get('service', ''),
                'code': request.POST.get('code', ''),
                'username': request.POST.get('username', ''),
                'password': request.POST.get('password', ''),

            }
        except KeyError:
            logger.warning('Got malformed POST request: %s' % str(request.POST))
            return HttpResponse('MALFORMED')

        status = PaymentProcessor.online(params, ip=request.META['REMOTE_ADDR'])
        return HttpResponse(status)


class ReturnView(DetailView):
    """
    This view just redirects to standard backend success or failure link.
    """
    model = Payment

    def post(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs)

    def render_to_response(self, context, **response_kwargs):
        if self.request.POST['status'] == 'OK':
            return HttpResponseRedirect(reverse('getpaid-success-fallback', kwargs={'pk': self.object.pk}))
        else:
            return HttpResponseRedirect(reverse('getpaid-failure-fallback', kwargs={'pk': self.object.pk}))



from django.conf.urls import patterns, url
from django.views.decorators.csrf import csrf_exempt
from getpaid.backends.przelewy24.views import OnlineView, SuccessView, FailureView

urlpatterns = patterns('',
    url(r'^online/$', csrf_exempt(OnlineView.as_view()), name='getpaid-przelewy24-online'),
    url(r'^success/(?P<pk>\d+)/', csrf_exempt(SuccessView.as_view()), name='getpaid-przelewy24-success'),
    url(r'^failure/(?P<pk>\d+)/', csrf_exempt(FailureView.as_view()), name='getpaid-przelewy24-failure'),

)



import logging
from celery.task.base import task
from django.db.models.loading import get_model

logger = logging.getLogger('getpaid.backends.przelewy24')


@task
def get_payment_status_task(payment_id, p24_session_id, p24_order_id, p24_kwota):
    Payment = get_model('getpaid', 'Payment')
    try:
        payment = Payment.objects.get(pk=int(payment_id))
    except Payment.DoesNotExist:
        logger.error('Payment does not exist pk=%d' % payment_id)
        return

    from getpaid.backends.przelewy24 import PaymentProcessor  # Avoiding circular import
    processor = PaymentProcessor(payment)
    processor.get_payment_status(p24_session_id, p24_order_id, p24_kwota)



from setuptools import find_packages, setup

with open('README.rst') as f:
    long_description = f.read()

setup(
    name='django-getpaid',
    description='Multi-broker payment processor for django',
    long_description=long_description,
    version='1.6.0',
    packages=find_packages(),
    url='https://github.com/cypreess/django-getpaid',
    license='MIT',
    author='Krzysztof Dorosz',
    author_email='cypreess@gmail.com',
    extras_require={
        'payu': [
            'django-celery>=3.0.11',
        ],
        'przelewy24': [
            'django-celery>=3.0.11',
            'pytz',
        ],
        'moip': [
            'requests',
        ],
    },

)



from django.conf.urls import patterns, url
from getpaid.backends.paymill.views import PaymillView

urlpatterns = patterns('',
    url(r'^payment/authorization/(?P<pk>[0-9]+)/$', PaymillView.as_view(), name='getpaid-paymill-authorization'),
)



import datetime
from decimal import Decimal
import hashlib
import logging
import urllib
from django.contrib.sites.models import Site
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse
from django.utils.timezone import utc
from django.utils.translation import ugettext_lazy as _
from getpaid import signals
from getpaid.backends import PaymentProcessorBase

logger = logging.getLogger('getpaid.backends.dotpay')


class DotpayTransactionStatus:
    STARTED = 1
    FINISHED = 2
    REJECTED = 3
    REFUNDED = 4
    RECLAMATION = 5


class PaymentProcessor(PaymentProcessorBase):
    BACKEND = 'getpaid.backends.dotpay'
    BACKEND_NAME = _('Dotpay')
    BACKEND_ACCEPTED_CURRENCY = ('PLN', 'EUR', 'USD', 'GBP', 'JPY', 'CZK', 'SEK')
    BACKEND_LOGO_URL = 'getpaid/backends/dotpay/dotpay_logo.png'

    _ALLOWED_IP = ('195.150.9.37', )
    _ACCEPTED_LANGS = ('pl', 'en', 'de', 'it', 'fr', 'es', 'cz', 'ru', 'bg')
    _GATEWAY_URL = 'https://ssl.dotpay.eu/'
    _ONLINE_SIG_FIELDS = ('id', 'control', 't_id', 'amount', 'email', 'service', 'code', 'username', 'password', 't_status')

    @staticmethod
    def compute_sig(params, fields, PIN):
        text = PIN + ":" + (u":".join(map(lambda field: params.get(field, ''), fields)))
        return hashlib.md5(text).hexdigest()

    @staticmethod
    def online(params, ip):

        allowed_ip = PaymentProcessor.get_backend_setting('allowed_ip', PaymentProcessor._ALLOWED_IP)

        if len(allowed_ip) != 0 and ip not in allowed_ip:
            logger.warning('Got message from not allowed IP %s' % str(allowed_ip))
            return 'IP ERR'

        PIN = PaymentProcessor.get_backend_setting('PIN', '')

        if params['md5'] != PaymentProcessor.compute_sig(params, PaymentProcessor._ONLINE_SIG_FIELDS, PIN):
            logger.warning('Got message with wrong sig, %s' % str(params))
            return 'SIG ERR'

        try:
            params['id'] = int(params['id'])
        except ValueError:
            return 'ID ERR'
        if params['id'] != int(PaymentProcessor.get_backend_setting('id')):
            return 'ID ERR'

        from getpaid.models import Payment
        try:
            payment = Payment.objects.get(pk=int(params['control']))
        except (ValueError, Payment.DoesNotExist):
            logger.error('Got message for non existing Payment, %s' % str(params))
            return 'PAYMENT ERR'

        amount, currency = params.get('orginal_amount', params['amount'] + ' PLN').split(' ')

        if currency != payment.currency.upper():
            logger.error('Got message with wrong currency, %s' % str(params))
            return 'CURRENCY ERR'

        payment.external_id = params.get('t_id', '')
        payment.description = params.get('email', '')

        if int(params['t_status']) == DotpayTransactionStatus.FINISHED:
            payment.amount_paid = Decimal(amount)
            payment.paid_on = datetime.datetime.utcnow().replace(tzinfo=utc)
            if payment.amount <= Decimal(amount):
                # Amount is correct or it is overpaid
                payment.change_status('paid')
            else:
                payment.change_status('partially_paid')
        elif int(params['t_status']) in [DotpayTransactionStatus.REJECTED, DotpayTransactionStatus.RECLAMATION, DotpayTransactionStatus.REFUNDED]:
            payment.change_status('failed')

        return 'OK'

    def get_URLC(self):
        urlc = reverse('getpaid-dotpay-online')
        current_site = Site.objects.get_current()
        if PaymentProcessor.get_backend_setting('force_ssl', False):
            return 'https://%s%s' % (current_site.domain, urlc)
        else:
            return 'http://%s%s' % (current_site.domain, urlc)

    def get_URL(self, pk):
        current_site = Site.objects.get_current()
        url = reverse('getpaid-dotpay-return', kwargs={'pk': pk})
        if PaymentProcessor.get_backend_setting('force_ssl', False):
            return 'https://%s%s' % (current_site.domain, url)
        else:
            return 'http://%s%s' % (current_site.domain, url)

    def get_gateway_url(self, request):
        """
        Routes a payment to Gateway, should return URL for redirection.
        """
        params = {
            'id': PaymentProcessor.get_backend_setting('id'),
            'description': self.get_order_description(self.payment, self.payment.order),
            'amount': self.payment.amount,
            'currency': self.payment.currency,
            'type': 0,  # show "return" button after finished payment
            'control': self.payment.pk,
            'URL': self.get_URL(self.payment.pk),
            'URLC': self.get_URLC(),
        }

        user_data = {
            'email': None,
            'lang': None,
        }
        signals.user_data_query.send(sender=None, order=self.payment.order, user_data=user_data)

        if user_data['email']:
            params['email'] = user_data['email']

        if user_data['lang'] and user_data['lang'].lower() in PaymentProcessor._ACCEPTED_LANGS:
            params['lang'] = user_data['lang'].lower()
        elif PaymentProcessor.get_backend_setting('lang', False) and \
                PaymentProcessor.get_backend_setting('lang').lower() in PaymentProcessor._ACCEPTED_LANGS:
            params['lang'] = PaymentProcessor.get_backend_setting('lang').lower()

        if PaymentProcessor.get_backend_setting('onlinetransfer', False):
            params['onlinetransfer'] = 1
        if PaymentProcessor.get_backend_setting('p_email', False):
            params['p_email'] = PaymentProcessor.get_backend_setting('p_email')
        if PaymentProcessor.get_backend_setting('p_info', False):
            params['p_info'] = PaymentProcessor.get_backend_setting('p_info')
        if PaymentProcessor.get_backend_setting('tax', False):
            params['tax'] = 1

        if PaymentProcessor.get_backend_setting('method', 'get').lower() == 'post':
            return self._GATEWAY_URL, 'POST', params
        elif PaymentProcessor.get_backend_setting('method', 'get').lower() == 'get':
            for key in params.keys():
                params[key] = unicode(params[key]).encode('utf-8')
            return self._GATEWAY_URL + '?' + urllib.urlencode(params), "GET", {}
        else:
            raise ImproperlyConfigured('Dotpay payment backend accepts only GET or POST')



from django.core.management.base import BaseCommand
from getpaid.backends.transferuj import PaymentProcessor


class Command(BaseCommand):
    help = 'Display URL path for Transferuj.pl Online URL configuration'

    def handle(self, *args, **options):

        key = PaymentProcessor.get_backend_setting('key', None)
        if key is None:
            self.stdout.write('Please be sure to provide "key" setting for this backend (random max. 16 characters)')
        else:
            self.stdout.write('Please setup in Transferuj.pl user defined key (for security signing): %s\n' % key)
